from common_utils.json_schema_methods import JsonSchemaMethods


def test_get_key_value_by_given_path():
    input_dict = {"a": {"b": {"c": 1}}}
    keys = ["a", "b", "c"]

    # Test case 1: Check returned value when as_pointer is False (default)
    returned_value = JsonSchemaMethods.get_key_value_by_given_path(input_dict, keys)
    expected_value = 1
    assert (
        returned_value == expected_value
    ), f"Expected {expected_value}, but got {returned_value}"

    # Test case 2: Check returned value when as_pointer is True
    returned_value = JsonSchemaMethods.get_key_value_by_given_path(
        input_dict, keys, as_pointer=True
    )
    expected_value = 1
    assert (
        returned_value == expected_value
    ), f"Expected {expected_value}, but got {returned_value}"



import copy
import logging

from jsonschema import ValidationError, validate


class JsonSchemaMethods:
    def custom_validate(_data: dict, _schema: dict) -> dict:
        """this method gets a json object and schema and checks if the json object fits the schema.
        - we use the validate function provided by the jsonschema module
        - if a missing key is found by the validate function it throws an exception (mentioning only the first missing key)
        - once we get this exception, we add the missing key to the json and set the value as defined in the given schema
        - we keep checking until the json match the schema
        - once there are no missing keys in the json, we return it

        if the schema is not valid - an exception is thrown
        if there is a type mismatch - an exception is thrown

        Args:
            _data (dict): the json object to validate
            _schema (dict): the schema

        Raises:
            e: the schema is not valid
            e: different data type
            e: other exception

        Returns:
            dict: the json object with all keys (according to the schema)
        """

        # deepcopy makes sure we won't change the original values we have in the record we get
        _data_copy = copy.deepcopy(_data)
        while True:
            try:
                validate(_data_copy, _schema)
                break
            except ValidationError as e:
                if "error_msg" in e.schema:
                    logging.error(e.schema["error_msg"])
                    raise e

                # the schema is valid, check the data
                else:
                    # missing key
                    if e.validator == "required":
                        missing_property = e.message.split(" ")[0].replace("'", "")
                        logging.warning(
                            f"{'.'.join(str(p) for p in e.path)}.{missing_property} property is missing"
                        )

                        # add nested missing property and set default value
                        if e.path:
                            path_to_missing_property = list(e.path)
                            # create a pointer to the location where the property is missing (in the json object to validate)
                            _missing_property_pointer = (
                                JsonSchemaMethods.get_key_value_by_given_path(
                                    input_dict=_data_copy,
                                    keys=path_to_missing_property,
                                    as_pointer=True,
                                )
                            )

                            # assign the default value of the nested missing property to the pointer of
                            # the json object (since it's a pointer, this will change the json object as well)
                            _missing_property_pointer[
                                missing_property
                            ] = JsonSchemaMethods.get_default_value_of_given_property(
                                schema=_schema,
                                property_name=missing_property,
                                property_path=path_to_missing_property,
                            )
                        # add NON nested missing property (first level in the schema) and set default value
                        else:
                            _data_copy[
                                missing_property
                            ] = JsonSchemaMethods.get_default_value_of_given_property(
                                schema=_schema,
                                property_name=missing_property,
                                property_path=[missing_property],
                            )

                    # different data type
                    elif e.validator == "type":
                        logging.critical(
                            f"the data type of {e.json_path} property is different between "
                            "the source and the target. Please check"
                        )
                        raise e
                    # other kind of failure
                    else:
                        raise e
        return _data_copy

    def get_key_value_by_given_path(
        input_dict: dict, keys: list, as_pointer: bool = False
    ) -> any:
        """
        Get the value of a dictionary attribute specified by a list of keys, either as a pointer or a deep copy.

        Args:
        - input_dict (dict): The input dictionary.
        - keys (list): A list of keys specifying the path to the desired dictionary attribute.
        - as_pointer (bool): If True, returns a pointer to the desired dictionary attribute. If False, returns a deep copy.

        Returns:
        - Any: The value of the desired dictionary attribute.
        """
        if as_pointer:
            dict_to_use = input_dict
        else:
            dict_to_use = copy.deepcopy(input_dict)

        for key in keys:
            dict_to_use = dict_to_use[key]

        return dict_to_use

    def get_default_value_of_given_property(
        schema: dict, property_name: str, property_path: list
    ) -> any:
        """
        Get the default value of a specified property in a JSON schema.

        Args:
        - schema (dict): The input JSON schema.
        - property_name (str): The name of the property to get the default value for.
        - property_path (list): A list of keys specifying the path to the desired property in the JSON schema.

        Returns:
        - Any: The default value of the specified property in the JSON schema.
        """
        path_to_property = [
            "properties",
            property_path[0],
            "default",
        ]

        # for nested properties
        if len(property_path) > 1:
            for p in property_path[1:]:
                path_to_property.append(p)

            path_to_property.append(property_name)

        return JsonSchemaMethods.get_key_value_by_given_path(
            input_dict=schema, keys=path_to_property
        )



import logging
from datetime import datetime, timedelta

from airflow.decorators import dag, task
from airflow.utils.dates import days_ago
from kubernetes.client import models as k8s

default_args = {
    "owner": "aiola-dev",
    "depends_on_past": False,
    "start_date": days_ago(0),
    "catchup": False,
    "email": ["aiola@example.com"],
    "email_on_failure": False,
    "email_on_retry": False,
    "retries": 0,
    "retry_delay": timedelta(minutes=5),
}


@dag(
    schedule="@monthly",
    start_date=datetime(2021, 12, 1),
    catchup=False,
    default_args=default_args,
)
def taskflow():
    @task
    def send_xcom():
        logging.info("HI HI HI")
        return "Hello XCOM"

    @task
    def print_msg(msg: str):
        logging.info(f"Msg: {msg}")

    print_msg(send_xcom())


dag = taskflow()



import json
import re


class GeneratePythonObjectFromSqlDataType:
    def generate_python_object_from_sql_data_type(self, sql_data_type: str) -> any:
        """this method takes a sql data type and returns the corresponding python object (empty)
        e.g. string will be returned as empty string "", arrat as empty list and so on

        Args:
            sql_data_type (str): a sql data type

        Returns:
            any: an empty python object that represents the sql data type
        """
        x = (
            sql_data_type.replace("string", '""')
            .replace("boolean", "False")
            .replace("bigint", "0")
            .replace("int", "0")
        )

        # replace decimal(x, y) with 0
        x = re.sub("decimal\(\d+(,\s\d+)\)", "0", x)

        # wrap each word followed by colon in double quotes (using regex backreferences: "\1" means the first capturing group)
        x = re.sub(r"(\w+):", r'"\1":', x)

        x = GeneratePythonObjectFromSqlDataType.replace_array_and_struct_keywords_with_suitable_brackets(
            x
        )

        # boolean value can't be converted to json
        x = False if x == "False" else json.loads(x)

        return x

    def replace_array_and_struct_keywords_with_suitable_brackets(col_type: str) -> str:
        """this method replace the array<> and struct<> keywords with their corresponding chars in python
        e.g. "array<>" becomes [] and "struct<>" becomes {}

        Args:
            col_type (str): a sql data type

        Returns:
            str: adjusted sql data type
        """
        # get the indices of each per of brackets
        brackets_indices = Helpers.find_parens(col_type)

        # keep running until there are no more "<" to replace ("<" is the begining of array/struct)
        while col_type.find("<") != -1:
            open_parens_ind = col_type.find("<")
            close_parens_ind = brackets_indices[open_parens_ind]

            # get the first index of array/struct
            x = re.search(r"(\w+)<", col_type).group()

            if x == "array<":
                col_type = Helpers.replace_str_index(col_type, close_parens_ind, "]")
                col_type = col_type.replace(x, "[", 1)
            elif x == "struct<":
                col_type = Helpers.replace_str_index(col_type, close_parens_ind, "}")
                col_type = col_type.replace(x, "{", 1)

            # recalculate the indices of the open and close brackets
            brackets_indices = Helpers.find_parens(col_type)

        return col_type


class Helpers:
    def find_parens(s: str, brackets_type: list = ["<", ">"]) -> dict:
        """this method get a string and returns a dict of the open and closing indcies of each pair of brackets in the given string

        Args:
            s (str): string to search for brackets
            brackets_type (list, optional): the type of open/close brackets to look for. Defaults to ['<', '>'].

        Raises:
            IndexError: missing closing bracket
            IndexError: missing opening bracket

        Returns:
            dict: indices of open/close brackets.
            the key is the opening bracket index, the value is the closing bracket index
        """
        indices = {}
        pstack = []

        for i, c in enumerate(s):
            if c == brackets_type[0]:
                pstack.append(i)
            elif c == brackets_type[1]:
                if len(pstack) == 0:
                    raise IndexError("No matching closing parens at: " + str(i))
                indices[pstack.pop()] = i

        if len(pstack) > 0:
            raise IndexError("No matching opening parens at: " + str(pstack.pop()))

        return indices

    def replace_str_index(text: str, index: int = 0, replacement: str = "") -> str:
        return text[:index] + replacement + text[index + 1 :]


class GenerateSqlDataType:
    def generate_sql_data_type(python_object: any) -> str:
        """the reverse action of generate_python_object() method.
        this method takes a python object and returns a string representation of it in the form of sql data type

        Args:
            python_object (any): any python object

        Returns:
            str: sql data type
        """
        x = json.dumps(python_object)

        # TODO: collect the default values per type and use them instead. e.g. if the default value of a number
        # is -1, we won't support it
        x = (
            x.replace('""', "string")
            .replace("false", "boolean")
            .replace("true", "boolean")
            .replace("0", "decimal(38, 9)")
            .replace("[", "array<")
            .replace("]", ">")
            .replace("{", "struct<")
            .replace("}", ">")
            .replace('"', "")
            .replace(" ", "")
        )

        return x



import logging
from datetime import datetime, timedelta

import awswrangler as wr


class dateTimeMethods:
    def get_timestamp(timestamp_string: str) -> datetime:
        """
        Returns a timestamp object from a given timestamp string.

        The timestamp string can be in the format 'YYYY-MM-DD' (date) or 'YYYY-MM-DD HH:MM:SS' (datetime).

        Args:
            timestamp_string (str): The timestamp string to be parsed.

        Returns:
            datetime: The timestamp object.

        Raises:
            ValueError: If the timestamp string is in an invalid format.
        """

        if not isinstance(timestamp_string, str):
            raise ValueError("timestamp_string should be string")
        formats = ["%Y-%m-%d", "%Y-%m-%d %H:%M:%S"]
        for f in formats:
            try:
                return datetime.strptime(timestamp_string, f)
            except ValueError:
                pass
        raise ValueError(f"Invalid timestamp string. Should be one of {formats}")

    def get_max_timestamp_from_athena(
        col_name: str,
        tbl_name: str,
        database: str,
        workgroup: str,
        default_from_timestamp: str = "2022-01-01",
        reduce_10_seconds: bool = True,
    ) -> datetime:
        """
        Queries Athena database for the maximum timestamp in a specified table, and returns the timestamp minus 10 seconds.
        If the table doesn't exist - the default from_timestamp is returned
        If the schema doesn't exist - we create it AND the default from_timestamp is returned

        Args:

        col_name (str): the name of the column to get the max timestamp from
        tbl_name (str): the name of the table in the Athena database
        database (str): the name of the Athena database
        workgroup (str): the name of the Athena workgroup to use for the query
        default_from_timestamp (str): the default timestmap to use if the table is empty / doens't exist
        Returns:

        datetime: the maximum timestamp from the specified table, minus 10 seconds OR the default from_timestamp
        """

        logging.info(f"get the max timestamp from {tbl_name}")
        try:
            df = wr.athena.read_sql_query(
                sql=f"""select coalesce(max({col_name}), timestamp'{default_from_timestamp}') as max_timestamp from {tbl_name}""",
                database=database,
                workgroup=workgroup,
            )

            max_timestamp = df["max_timestamp"][0]

        except Exception as e:
            if "does not exist" in str(e):
                if "Table" in str(e):
                    logging.warning(
                        f"the table/view {database}.{tbl_name} doesn't exist"
                    )
                elif "Schema" in str(e):
                    logging.warning(
                        f"the schema {database} doesn't exists. Creating it now..."
                    )
                    wr.athena.start_query_execution(f"create database {database}")
                    logging.info(f"schema {database} was created")
                else:
                    logging.warning(
                        f"the query to get max timestamp from {database}.{tbl_name} failed with this error message: {e}"
                    )
            else:
                logging.warning(
                    f"the query to get max timestamp from {database}.{tbl_name} failed with this error message: {e}"
                )

            logging.info(
                "from_timestamp will be assigned with default value to fetch all data"
            )
            max_timestamp = default_from_timestamp

        # we reduce 10 seconds to make sure we don't miss any rows
        if reduce_10_seconds:
            return max_timestamp - timedelta(seconds=10)
        else:
            return max_timestamp



class StringTransformations:
    def recursion_lower(self, x: any) -> any:
        """this method transform all characters of the given object to lower case

        Args:
            x (any): any python object

        Returns:
            any: the python object after lower transformation
        """
        if type(x) is str:
            return x.lower()
        elif type(x) is list:
            return [self.recursion_lower(i) for i in x]
        elif type(x) is dict:
            return {
                self.recursion_lower(k): self.recursion_lower(v) for k, v in x.items()
            }
        else:
            return x



import argparse
import logging
from datetime import datetime

import awswrangler as wr
from dateutil.relativedelta import relativedelta

from common_utils.date_time_methods import dateTimeMethods


class dataLoadingManagement:
    """
    Class to manage data loading operations. It provides functions to set the start and end timestamps for the data loading,
    validating the start and end timestamps, and creating the schema if the data is being loaded for the first time.

    Args:
        app_args: An instance of argparse.Namespace, containing command line arguments passed to the application.
        timestamp_col_name: A string, representing the name of the column in the source data which contains the timestamp.
        tbl_to_check_max_timestamp: A string, representing the name of the table in the destination database to check for the max timestamp.
        from_and_to_as_dates: A boolean, indicating whether the start and end timestamps should be set as dates (midnight of the given date).
        create_schema_if_initial_load: A boolean, indicating whether the schema should be created if the data is being loaded for the first time.

    Attributes:
        given_from_timestamp: A string, representing the start timestamp specified by the user.
        given_to_timestamp: A string, representing the end timestamp specified by the user.
        initial_load: A string, indicating whether the data is being loaded for the first time.
        initial_load_from_timestamp: A string, representing the start timestamp to use if the data is being loaded for the first time and the start timestamp is specified by the user.
        initial_load_from_timestamp_months_back: An integer, representing the number of months to go back to calculate the default start timestamp if the data is being loaded for the first time and the start timestamp is not specified by the user.
        dest_schema: A string, representing the name of the destination database.
        timestamp_col_name: A string, representing the name of the column in the source data which contains the timestamp.
        tbl_to_check_max_timestamp: A string, representing the name of the table in the destination database to check for the max timestamp.
        default_from_timestamp: A datetime object, representing the default start timestamp to use if the data is being loaded for the first time and the start timestamp is not specified by the user.
        from_timestamp: A datetime object, representing the start timestamp to use for the data loading.
        to_timestamp: A datetime object, representing the end timestamp to use for the data loading.
    """

    def __init__(
        self,
        app_args: argparse.Namespace,
        timestamp_col_name: str,
        tbl_to_check_max_timestamp: str,
        from_and_to_as_dates: bool = False,
        create_schema_if_initial_load: bool = True,
    ):
        """
        Initializes the class and sets the start and end timestamps, and creates the schema if the data is being loaded for the first time.
        """
        self.given_from_timestamp = app_args.from_timestamp
        self.given_to_timestamp = app_args.to_timestamp
        self.initial_load = app_args.initial_load
        self.initial_load_from_timestamp = app_args.initial_load_from_timestamp
        self.initial_load_from_timestamp_months_back = (
            app_args.initial_load_from_timestamp_months_back
        )
        self.dest_schema = app_args.destination_database_name
        self.athena_workgroup = app_args.athena_workgroup
        self.timestamp_col_name = timestamp_col_name
        self.tbl_to_check_max_timestamp = tbl_to_check_max_timestamp

        # to be used if nothing else is given
        self.default_from_timestamp = (
            datetime.now()
            - relativedelta(months=self.initial_load_from_timestamp_months_back)
        ).replace(microsecond=0)

        self.from_timestamp = None
        self.to_timestamp = None

        self.set_from_and_to_timestamps(as_dates=from_and_to_as_dates)

        if create_schema_if_initial_load:
            self.create_schema_if_not_exists()

    def set_from_and_to_timestamps(self, as_dates: bool = False):
        """
        Sets the start and end timestamps for the data loading based on the user-specified timestamps and default values.

        Args:
            as_dates: A boolean, indicating whether the start and end timestamps should be set as dates (midnight of the given date).

        Returns:
            None
        """
        # set from/to_timestamp to fetch all data when the table is recrated
        if self.initial_load:
            self.set_from_timestamp_initial_load()

            self.to_timestamp = datetime.now()
        else:
            self.set_from_timestamp_partial_load()

            if self.given_to_timestamp:
                self.to_timestamp = dateTimeMethods.get_timestamp(
                    self.given_to_timestamp
                )
            else:
                self.to_timestamp = datetime.now()

        # set the from and to timestamps as midnight of the given date
        if as_dates:
            self.from_timestamp = self.from_timestamp.replace(
                hour=0, minute=0, second=0, microsecond=0
            )
            self.to_timestamp = self.to_timestamp.replace(
                hour=0, minute=0, second=0, microsecond=0
            )

        self.validate_from_and_to_timestamp()

    def set_from_timestamp_initial_load(self):
        """Sets the from_timestamp for initial load.

        If `initial_load_from_timestamp` is given, it is used as the `from_timestamp`.
        If not, the `from_timestamp` is calculated as `initial_load_from_timestamp_months_back` months back from today.
        The information about the process is logged.

        Returns:
            None
        """
        logging.info(
            "initial_load is True, so from_timestamp and to_timestamp will be assigned with default values to fetch all data"
        )
        if self.initial_load_from_timestamp:
            logging.info(
                f"initial_load_from_timestamp is given and will be used as the from_timestamp: {self.initial_load_from_timestamp}"
            )
            self.from_timestamp = dateTimeMethods.get_timestamp(
                self.initial_load_from_timestamp
            )
        else:
            self.from_timestamp = self.default_from_timestamp
            logging.info(
                f"initial_load_from_timestamp was NOT given, from_timestamp will be {self.initial_load_from_timestamp_months_back} month back from today: {self.from_timestamp}"
            )

    def set_from_timestamp_partial_load(self):
        """Sets the from_timestamp for partial load.

        If `given_from_timestamp` is given, it is used as the `from_timestamp`.
        If not, the `from_timestamp` is calculated as the maximum timestamp in the `tbl_to_check_max_timestamp` table in the Athena database.
        The default value is set if it can't be retrieved from the table.

        Returns:
            None
        """
        if self.given_from_timestamp:
            self.from_timestamp = dateTimeMethods.get_timestamp(
                self.given_from_timestamp
            )
        else:
            self.from_timestamp = dateTimeMethods.get_max_timestamp_from_athena(
                col_name=self.timestamp_col_name,
                tbl_name=self.tbl_to_check_max_timestamp,
                database=self.dest_schema,
                workgroup=self.athena_workgroup,
                default_from_timestamp=self.default_from_timestamp,
                reduce_10_seconds=False,
            )

    def validate_from_and_to_timestamp(self):
        """
        Validate the from and to timestamps to make sure that the to_timestamp is not smaller than the from_timestamp.
        Log a warning if the to_timestamp is smaller, and log an info message if the validation is successful.

        Returns:
            None
        """
        if self.to_timestamp < self.from_timestamp:
            logging.warning(
                "to_timestamp is bigger than the from_timestamp. no data to fetch"
            )
        else:
            logging.info(
                f"fetching data from: {self.from_timestamp} until: {self.to_timestamp}"
            )

    def create_schema_if_not_exists(self):
        """
        Check if the "initial_load" attribute is set to True and create the destination schema if it doesn't exist yet.

        Returns:
            None
        """
        if self.initial_load:
            logging.info(
                f"initial load is True. the schema {self.dest_schema} will be created if not exist yet"
            )
            wr.athena.start_query_execution(
                f"create database if not exists {self.dest_schema}"
            )



from datetime import timedelta

from airflow import DAG
from airflow.contrib.operators.kubernetes_pod_operator import KubernetesPodOperator
from airflow.operators.dummy_operator import DummyOperator
from airflow.utils.dates import days_ago

default_args = {
    'owner': 'Airflow',
    'depends_on_past': False,
    'start_date': days_ago(0),
    'catchup': False,
    'email': ['airflow@example.com'],
    'email_on_failure': False,
    'email_on_retry': False,
    'retries': 0,
    'retry_delay': timedelta(minutes=5),
}

dag = DAG(
    'dag_that_executes_via_KubernetesPodOperator',
    default_args=default_args,
    schedule_interval=timedelta(minutes=30),
    max_active_runs=1,
    concurrency=10
)

# Generate 2 tasks
tasks = ["task{}".format(i) for i in range(1, 3)]
example_dag_complete_node = DummyOperator(task_id="example_dag_complete", dag=dag)

org_dags = []
for task in tasks:

    bash_command = 'echo HELLO'

    org_node = KubernetesPodOperator(
        namespace='default',
        image="python",
        cmds=["python", "-c"],
        arguments=["print('HELLO')"],
        labels={"foo": "bar"},
        image_pull_policy="Always",
        name=task,
        task_id=task,
        is_delete_operator_pod=False,
        get_logs=True,
        dag=dag
    )

    org_node.set_downstream(example_dag_complete_node)


from datetime import timedelta

from airflow.contrib.operators.kubernetes_pod_operator import KubernetesPodOperator
from airflow.utils.dates import days_ago
from kubernetes.client import models as k8s

from airflow import DAG

default_args = {
    "owner": "aiola-dev",
    "depends_on_past": False,
    "start_date": days_ago(0),
    "catchup": False,
    "email": ["aiola@example.com"],
    "email_on_failure": False,
    "email_on_retry": False,
    "retries": 0,
    "retry_delay": timedelta(minutes=5),
}

dag = DAG(
    "gad_flow",
    default_args=default_args,
    schedule_interval=timedelta(minutes=30),
    max_active_runs=1,
    concurrency=10,
)
environment = [
    k8s.V1EnvVar(name="PROJECTDIR", value="value1"),
    k8s.V1EnvVar(name="key2", value="value2"),
]
volume_mount = k8s.V1VolumeMount(
    name="project-volume",
    mount_path="/opt/aiola/projects",
    sub_path=None,
    read_only=True,
)

volume = k8s.V1Volume(
    name="project-volume",
    # persistent_volume_claim=k8s.V1PersistentVolumeClaimVolumeSource(claim_name="project-volume"),
    host_path=k8s.V1HostPathVolumeSource(path="/home/docker/projects"),
)
schema_checker = KubernetesPodOperator(
    volumes=[volume],
    volume_mounts=[volume_mount],
    env_vars={"PYTHONPATH": "/opt/aiola/projects/data-utils"},
    namespace="default",
    image="gad-papermill:0.1",
    cmds=["python"],
    arguments=[
        "/opt/aiola/projects/data-utils/inspection_streaming/schema_checker_for_federation.py",
        "--table_name",
        "{{ var.value.table_to_check }}",
        "--table_schema_file_path",
        "/opt/aiola/projects/data-utils/inspection_streaming/tyson/ddb_schemas/ddb_schema_items.json",
        "--aws_region",
        "eu-west-1",
    ],
    labels={"owner": "dev"},
    image_pull_policy="Never",
    name="schema_checker",
    task_id="schema_checker-test",
    is_delete_operator_pod=True,
    get_logs=True,
    dag=dag,
)
great_expectation_task = KubernetesPodOperator(
    namespace="default",
    image="gad-ge:0.1",
    cmds=["great_expectations", "init", "--help"],
    arguments=["print('HELLO')"],
    labels={"foo": "bar"},
    image_pull_policy="Never",
    name="great_expectation_task",
    task_id="great_expectation_task",
    is_delete_operator_pod=True,
    get_logs=True,
    dag=dag,
)
dbt_task = KubernetesPodOperator(
    volumes=[volume],
    volume_mounts=[volume_mount],
    # env_vars={"DBT_PROFILES_DIR": "/opt/aiola/projects/dbt/profiles"},
    namespace="default",
    image="gad-dbt:0.1",
    cmds=["dbt", "ls"],
    arguments=[
        "--project-dir",
        "/opt/aiola/projects/strauss-dbt-poc",
        "--profiles-dir",
        "/opt/aiola/projects/strauss-dbt-poc",
    ],
    labels={"Task": "DBT"},
    image_pull_policy="Never",
    name="run_dbt",
    task_id="run_dbt",
    is_delete_operator_pod=True,
    get_logs=True,
    dag=dag,
)

schema_checker >> great_expectation_task >> dbt_task
# schema_checker.dry_run()
